function combinedImg = overlay_w_transparency(anat, funct, thresholded_functmap, opacity, newColormap)
% DESCRIPTION: Overlay functional images on anatomical images.
% This function overlays functional (e.g. T-maps) on top of anatomical 
% images.
% INPUT PARAMETERS:
%     anat:   3D anatomical image (or underlay)
%     funct:  3D functional image (or overlay)
%     threshold:  scalar value applied to the functional image.  values above the
%                 threshold are retained, values below are discarded.
%     slices: the slice (or vector of slices) to display.
%     orientation: slice orientation (takes values: 'axial' (or 'transverse'),
%                  'sagittal', and 'coronal'.
%     opacity:   scalar quantity in [0,1] which sets the transparency, with 
%                0 being fully transparent).
%--------------------------------------------------------------------------
% by DuBois Bowman, Emory University, January 28, 2009 [Adapted from R. Raizada]
%--------------------------------------------------------------------------
% The approach taken here specifies a real RGB (red-green-blue) color value
% for every voxel in the two images, allowing transparency in the overlay 
% by making a weighted-combination of the anatomical and functional images.
% The RGB approach also provides flexibility by not using a colormap, since 
% MATLAB only allows 1 colormap per image.
%
% To figure out what RGB values to give, we read in the RGB values
% from existing MATLAB colormaps, which are 256x3 matrices.
%
% The MATLAB 'image' function (unscaled, i.e. not 'imagesc'), takes each 
% matrix-element, sees what whole-number from 1 to 256 its closest to, and 
% looks up the values in that row of the colormap matrix. The three values 
% in the corresponding row are the RGB values for the respective color.
%
% The trick that we'll use below will be to scale our anatomical and
% functional images from 1 to 256, then read in the RGB values from the 
% corresponding rows of the gray (for anatomical) and jet (for functional) 
% colormap matrices. Then we can make a weighted sum of these two images.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modified by: Lijun Zhang, Ph.D                   %
% Center for Biomedical Imaging Statistics (CBIS)  %
% Dept. of Biostatistics and Bioinformatics        %
% Emory University                                 %
% Atlanta, GA, 30322                               %
% Email: l.zhang@emory.edu                         %
% http://www.sph.emory.edu/bios/CBIS/about.html    %
% Last revision: Jan 25, 2010                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

%------- Perform some initial checks ------------
imdim=size(anat);
imdim0=size(funct);
if sum(imdim==imdim0)<3 % ensure funct and anat images have same dimensions.
    error('Anatomical and functional images MUST be of the same size')
end
%--------------------------------------------------------------------------
% Make full RGB-images by looking up the corresponding rows of the gray and 
% jet colormaps. A 2D RGB-image is stored in a 3D-matrix, with 3 "color 
% slabs" along the 3rd dimension.  The slabs represent images full of Red, 
% Green, and Blue values, respectively.  Since we are working on a 3D image, 
% we will create a "4D" structure, with each element (indexed by RGB_dim 
% below) representing an R, G, or B slab of a 3D image.
for RGB_dim = 1:3
    RGB(RGB_dim).anat = zeros(imdim); %%% initialize
    RGB(RGB_dim).funct = zeros(imdim);
    combinedImg(RGB_dim).compound = zeros(imdim);
end

% Get Matlab's gray and jet colormap matrices.

%---Look up and store the appropriate RGB values. ----------------------
for RGB_dim = 1:3,  % Loop through the three slabs: R, G, and B
    % Each entry in the 1-to-256-scaled anatomical (functional) matrix gives 
    % a row in the colormap matrix to look up.  The three elements of that 
    % row gets assigned to the respective color slab.  Note that we 
    % actually look up all the rows at once, returning a vector which then
    % is reshaped to the appropriate size of the image.
    color_slab_vals_for_anat = newColormap(anat, RGB_dim);
    color_slab_vals_for_funct = newColormap(funct, RGB_dim);
    RGB(RGB_dim).anat = reshape(color_slab_vals_for_anat, imdim);
    RGB(RGB_dim).funct = reshape(color_slab_vals_for_funct, imdim);
    
    % Make a compound image as a weighted sum of the anatomical image and the 
    % functional image. The more weighting we give to the functional image, 
    % the more opaque it will look. (opacity in [0, 1], with 0 being 
    % fully transparent).
    % Below the functional threshold, we only keep the anatomical's values.
    % Above the threshold, we take a weighted sum of the functional  RGB
    % values and the anatomical's RGB values.  'min' ensures RGB<=1.
    combinedImg(RGB_dim).combound=min(1, ...
        (thresholded_functmap==0) .* ...    % below threshold
            RGB(RGB_dim).anat  + ...  
        (thresholded_functmap>0).* ...      % above threshold
            ( (1-opacity) * RGB(RGB_dim).anat + ...
               opacity * RGB(RGB_dim).funct ));
end;  % End of loop through the RGB dimension. 

