classdef BayesianSpatialModel < handle
%BAYES Summary of this class goes here
% by Sanjay Agravat, Emory University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modified by: Lijun Zhang, Ph.D                   %
% Center for Biomedical Imaging Statistics (CBIS)  %
% Dept. of Biostatistics and Bioinformatics        %
% Emory University                                 %
% Atlanta, GA, 30322                               %
% Email: l.zhang@emory.edu                         %
% http://www.sph.emory.edu/bios/CBIS/about.html    %
% Jan, 2009, Last revision: Jan 25, 2010           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    properties
        CombinedRegions
        monteCarloParams
        gammaParams
        Y
        X
        X_Covariates
        BrainLocs
        MapRegions
        roi        
        mBigj
        G
        glocs
        vox
        estimates
        stimulus
        ContSim
        Pp
        PostSim1
        PostSim2
        Subject    
        NumStimulus
    end

%     properties (Dependent)
%         NumStimulus
%     end
    
    methods
        function obj=BayesianSpatialModel(gammaParams, monteCarloParams, CombinedRegions, analysis_data)
            load(analysis_data)
            obj.monteCarloParams = monteCarloParams;
            obj.gammaParams = gammaParams;
            obj.Y = Y;
            obj.X = X;
            obj.X_Covariates = X_Covariates;
            obj.BrainLocs = brain_locs;
            obj.CombinedRegions = CombinedRegions;
            obj.MapRegions = map_regions;
            obj.ContSim = [];
            obj.Pp = [];
            obj.PostSim1 = [];
            obj.PostSim2 = [];
            obj.Subject = [];
            obj.NumStimulus = [];
        end

        function estimate(obj)
            tic
            obj.createRegionLocations();
            obj.calcMarkovChainMonteCarlo();
            toc
        end

        function createRegionLocations(obj)
            regions = obj.CombinedRegions;
            num_stimulus = size(obj.X, 2);

            roi.mapRegions        = obj.MapRegions(obj.BrainLocs);
            roi.labs              = unique([roi.mapRegions]);
            roi.labs(roi.labs==0) = []; %remove non-brain label
     
            lastVox = 0;
            
            % G is the number of regions
            G = size(regions, 1);
            
            % initialize the Voxels for region g
            roi.numVoxels = zeros(1, G);
            
            vox = []; %% add by Lj on Dec 15, 2009;           
            regionLocations = [];%% add by Lj on Feb 22, 2010;  
                        
            % loop through the selected regions and merge selected clusters and
            % re-number            
            for g=1:G 
               
                % find locations within the user's selected regions 
                locs = find(ismember(roi.mapRegions, regions(g, :)));
                
                % num voxels at region of index (g)
                roi.numVoxels(g) = length(locs);
                
                % set the locations to correspond to a region
                roi.mapRegions(locs) = g;
                
                % the locations that are included for region(g)
                regionLocations(g).roi = locs;
                
                vox(g).indx = lastVox + 1 : lastVox + length(locs);

                lastVox = lastVox + length(locs);
                
                estimates = struct;
                
                % calculate the means/variances for each stimulus from the
                % estimates from Stage I
                for j=1:num_stimulus
                    % the index of subjects from each group
                    stimulus(j).subjects = find(obj.X(:,j)); 
                    
                    subjectsByLocation = obj.Y(stimulus(j).subjects, locs);
                    
                    % mean across voxels in region g for each subject
                    % returns column vector for the means of each row
                    mBigj(j).B(:,g)           = mean(subjectsByLocation, 2); %

                    % mean across subjects for each region g
                    % the mean of the row vector mean which returns a
                    % single result
                    mBgj(j).B(g)              = mean(mean(subjectsByLocation)); %

                    % mean across subjects
                    % returns row vector for the means of each column
                    mBvgj(j).B(vox(g).indx)   = mean(subjectsByLocation); 
                    
                    % variance across voxels in region g for each subject
                    vBigj_v(j).B(vox(g).indx) = var(subjectsByLocation); %variance across subjects
                    
                    % ???
                    vBgj(j).B(:,g)            = mean(var(subjectsByLocation));
                    
                    % initialize, automatically mean centers data
                    estimates(j).iGamma=inv(cov([mBigj(j).B])); 

                end
            end
            disp('saving observed data')
            save 'results/obs_dat.mat' mBvgj mBigj mBgj vBigj_v vBgj;
            
            obj.roi     = roi;
            obj.mBigj   = mBigj;
            obj.G       = G;
            obj.glocs   = regionLocations;
            obj.vox     = vox;
            obj.estimates = estimates;
            obj.stimulus  = stimulus;
        end

        function calcMarkovChainMonteCarlo(obj)
            disp('calcMarkovChainMonteCarlo');
            mcmc = MarkovChainMonteCarlo(obj);
            [post_sim1, post_sim2, numStimulus, M, NumRegions, SubjectId] = mcmc.gibbsSampler();
            
            obj.ContSim = mcmc.cont_sim;            
            obj.Pp = mcmc.pp;
            obj.PostSim1 = post_sim1;
            obj.PostSim2 = post_sim2;
            obj.Subject = SubjectId;
            obj.NumStimulus = numStimulus;
        end

    end
end
