classdef MarkovChainMonteCarlo < handle
%MARKOVCHAINMONTECARLO calculate the MCMC using Gibbs sampling
%   Detailed explanation goes here
% by Sanjay Agravat, Emory University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modified by: Lijun Zhang, Ph.D                   %
% Center for Biomedical Imaging Statistics (CBIS)  %
% Dept. of Biostatistics and Bioinformatics        %
% Emory University                                 %
% Atlanta, GA, 30322                               %
% Email: l.zhang@emory.edu                         %
% http://www.sph.emory.edu/bios/CBIS/about.html    %
% Jan, 2009, Last revision: Jan 25, 2010           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

    properties
        a0
        b0
        c0
        d0
        e0
        f0
        r0 
        s0
        thinFactor
        bayesSpatialModel  
        cont_sim
        bf
        pp
        post_sim1

    end

    methods
        function obj=MarkovChainMonteCarlo(hObj)
            obj.a0 = hObj.gammaParams.a0;
            obj.b0 = hObj.gammaParams.b0;
            obj.c0 = hObj.gammaParams.c0;
            obj.d0 = hObj.gammaParams.d0;
            obj.e0 = hObj.gammaParams.e0;
            obj.f0 = hObj.gammaParams.f0;
            obj.r0 = hObj.gammaParams.r0;
            obj.s0 = hObj.gammaParams.s0;
            obj.thinFactor = hObj.monteCarloParams.thinFactor;
            obj.bayesSpatialModel = hObj;

        end

        function [post_sim1, post_sim2, NumStimulus, M, NumRegions, SubjectId] = gibbsSampler(obj)
            bayesSpatialModel = obj.bayesSpatialModel;
            NumStimulus = size(bayesSpatialModel.X, 2);
            estimates = bayesSpatialModel.estimates;
            a0 = obj.a0;
            b0 = obj.b0;
            c0 = obj.c0;
            d0 = obj.d0;
            e0 = obj.e0;
            f0 = obj.f0;
            r0 = obj.r0;
            s0 = obj.s0;
            V=sum(bayesSpatialModel.roi.numVoxels);
        
            thin_factor= obj.thinFactor;
            count=0;
            prop_corrections=0;
            prop_corrections2=0;

            mY=mean(bayesSpatialModel.Y);

            mu0=mean(mY); % set a single prior value for all voxels, e.g. overall grand mean across voxels
            mu=single(full(logical(sparse(V,1))));

            M           = bayesSpatialModel.monteCarloParams.iterations + bayesSpatialModel.monteCarloParams.burnIn;            
            cov_wght    = bayesSpatialModel.monteCarloParams.covWeight;
            NumRegions  = bayesSpatialModel.G;
            glocs       = bayesSpatialModel.glocs;
            roi         = bayesSpatialModel.roi;
            Y           = bayesSpatialModel.Y;
            X_Covariates = bayesSpatialModel.X_Covariates;
            Q = size(X_Covariates,2); %%%covariant
            
            vox         = bayesSpatialModel.vox;
            mBigj       = bayesSpatialModel.mBigj;
            SubjectId   = zeros(NumStimulus,1);
            stimulus    = bayesSpatialModel.stimulus;      

            post_sim1 = zeros(0, NumStimulus);
            post_sim2 = zeros(0, NumStimulus);
            
            %%%%%find the mininum group to decide the Gama distribution
            temp = unique(bayesSpatialModel.X(:,1));
            for k = 1:length(temp)
                LenSub(k) = length(find(bayesSpatialModel.X(:,1)==temp(k)));
            end          
            minSubjects = min(LenSub);
            if minSubjects <= NumRegions
                wishartPara = 1;
            else
                wishartPara = 0;
            end
            
            for j=1:NumStimulus;
                SubjectId(j) = length(stimulus(j).subjects);
                               
                save_iter=0;
                h0 = SubjectId(j); % Wishart df (most diffuse possible for a proper prior)
                H_0b = inv(estimates(j).iGamma); % Group-specific prior for Gamma
                H_0 = cov_wght*diag(diag(H_0b)) + (1-cov_wght)*H_0b;  % take weighted combination -- shrinking toward independence (try several cov_wght's -- think about GOF criterion for selecting cov_wght);
                for m = 1 : M;
                    progressbar((m+((j - 1) * M))/(NumStimulus*M));
                    
                    if (m == 1 || mod(m,100)==0) && m<= bayesSpatialModel.monteCarloParams.burnIn
                        disp(['Starting Burn-In iteration ' num2str(m) '(Effect #' num2str(j) ' of ' num2str(NumStimulus) ')'])
                    elseif ((m-bayesSpatialModel.monteCarloParams.burnIn) == 1 || mod(m,100)==0) && m > bayesSpatialModel.monteCarloParams.burnIn
                        disp(['Starting iteration ' num2str(m-bayesSpatialModel.monteCarloParams.burnIn) '(Effect #' num2str(j) ' of ' num2str(NumStimulus) ')'])
                    end
                    
                    omega = zeros(NumRegions,1);
                    for g = 1 : NumRegions
                        count = count+1;
                        NumVoxelsInRegion = roi.numVoxels(g);%%% number of voxels in region g;
                        %set_seed(5*count);
                        if m == 1 % intialize
                            estimates(j).alpha = single(full(logical(sparse(SubjectId(j),NumRegions))));
                            sigma2g = 10 * max(var(Y(stimulus(j).subjects, [glocs(g).roi]))); % inflate initial variance
                            lambda2g = 5 * sigma2g/length(stimulus(j).subjects);
                            
                            if ~isempty(X_Covariates),
                                estimates(j).r([vox(g).indx],1:Q)=zeros(NumVoxelsInRegion,Q); % NumVoxelsInRegion x Q
                                tau2gq(1:Q)=5*sigma2g/length(stimulus(j).subjects); % tau2gq 1 by Q
                            end
                        else
                            lambda2g = 1/estimates(j).ilambda2(g);
                            sigma2g = 1/estimates(j).isigma2(g);
                            
                            if ~isempty(X_Covariates)
                                for q=1:Q
                                    tau2gq(q)=1/estimates(j).itau2gq(g,q);
                                end
                            end
                        end                        

                        %----- mu -------
%                        [Bigj, mu, estimates] = obj.calcMean(stimulus(j), estimates(j), glocs(g), sigma2g, lambda2g, SubjectId(j), vox(g), m);
                        Bigj = Y(stimulus(j).subjects,[glocs(g).roi]);%%beta for subjects in group j, and voxels in region g; SubjectId(j) x NumVoxelsInRegion
                        Bgj_bar_k = mean(Bigj);
                        
                        if ~isempty(X_Covariates) % the covariates
                            r_cov=estimates(j).r([vox(g).indx],:)*X_Covariates(stimulus(j).subjects,:)'; %NumVoxelsInRegion x SubjectId(j) (Vg xKj)
                            %r_cov_bar_k=mean(estimates(j).r([vox(g).indx],:)*X_Covariates(stimulus(j).subjects,:)',2)';% r Vg by Q,  X_cov Kj by Q, finally 1 x Vg
                            r_cov_bar_k=mean(r_cov,2)';% r Vg by Q,  X_cov Kj by Q, finally 1 x Vg
                        
                        else
                            r_cov=0;
                            r_cov_bar_k=0;
                        end                        
                        
                        omega(g) = (1/lambda2g + SubjectId(j)/sigma2g)^(-1);        %Covariance matrix:  Omega=omega*eye(numVoxels);
                        if m == 1
                            estimates(j).mu([vox(g).indx]) = Bgj_bar_k;
                        else
                            alpha_gj_bar_k = mean(estimates(j).alpha(:,g));
                            estimates(j).mu([vox(g).indx]) = normrnd(omega(g)*(mu0/lambda2g + SubjectId(j)*(Bgj_bar_k - alpha_gj_bar_k - r_cov_bar_k)/sigma2g),sqrt(omega(g)));
                        end

                        mu(vox(g).indx) = estimates(j).mu([vox(g).indx]);
                        estimates(j).theta(g) = mean(estimates(j).mu([vox(g).indx]));  % compute cluster means

                        %------r the covariate parameters------
                        if ~isempty(X_Covariates)% if there are any covariates...
                            if m==1
                                estimates(j).r([vox(g).indx],1:Q)=zeros(NumVoxelsInRegion,Q); % Vg x Q
                            else
                                for q=1:Q,
                                    r_nq=estimates(j).r([vox(g).indx],:);
                                    X_cov_nq=X_Covariates(stimulus(j).subjects,:);
                                    r_nq(:,q)=[];
                                    X_cov_nq(:,q)=[];

                                    r_cov_nq=(r_nq*X_cov_nq')'; %Kj x Vg, mean r*x without q.....

                                    phi(g,q)=(1/tau2gq(q) + sum(X_Covariates(stimulus(j).subjects,q).^2)/sigma2g)^(-1);        %Covariance matrix:  Phi=phi*eye(Vg);
                                    estimates(j).r([vox(g).indx],q)=normrnd(phi(g,q)*((X_Covariates(stimulus(j).subjects,q)'*(Bigj-repmat(estimates(j).alpha(:,g),1,NumVoxelsInRegion) -repmat(mu([vox(g).indx]),1,SubjectId(j))' -r_cov_nq))/sigma2g),sqrt(phi(g,q)));
                                       % Vg x 1                                                % 1 x Kj                           Kj x Vg                                                  G x Vg ( repeat alpha Vg times)        Kj x Vg     
                                end
                            end

                            estimates(j).rmean(g,:)=mean(estimates(j).r([vox(g).indx],:)); % intra region 1 x Q
                        end
            
                        %----- lambda -------
                        L1 = (estimates(j).mu(vox(g).indx) - mu0)*(estimates(j).mu(vox(g).indx)' - mu0);
                        estimates(j).ilambda2(g) = gamrnd(c0+NumVoxelsInRegion/2, (1/d0 + 0.5*L1)^(-1));

                        %------tau-----------
                        if ~isempty(X_Covariates)% if there are any covariates...
                            for q=1:Q
                                estimates(j).itau2gq(g,q)=gamrnd(e0+NumVoxelsInRegion/2, (1/f0 + 0.5* estimates(j).r(vox(g).indx,q)'*estimates(j).r(vox(g).indx,q))^(-1));
                            end
                        end
                        
                        %----- sigma -------
                        rigj = Bigj - repmat(estimates(j).alpha(:,g),1,NumVoxelsInRegion) -  repmat(estimates(j).mu(vox(g).indx),SubjectId(j),1)-r_cov';%Kj x Vg  changed last term
                        Ugj = sum(sum(rigj.^2,2)); % sum_i uigj
                        estimates(j).isigma2(g) = gamrnd(a0 + SubjectId(j)*NumVoxelsInRegion/2, 1/(1/b0 + 0.5*Ugj));
                        
                        %----- save lambda and sigma -------
                        if rem(m,thin_factor) == 0 && m > bayesSpatialModel.monteCarloParams.burnIn
                            if g == 1
                                save_iter = save_iter+1;
                            end
                            post_sim1(j).ilambda2(g,save_iter) = estimates(j).ilambda2(g);
                            post_sim1(j).isigma2(g,save_iter) = estimates(j).isigma2(g);
                            
                            if ~isempty(X_Covariates)
                                post_sim1(j).itau2gq(g,1:Q, save_iter)= estimates(j).itau2gq(g,1:Q);%%% RETHINK
                            end
                        end                        
                    end

                    
                    %----- Gamma -------
                    H1 = 0;
                    % for i = 1 : SubjectId(j)
                    %     H1 = H1+estimates(j).alpha(i,:)'*estimates(j).alpha(i,:);
                    % end
                    H1 = estimates(j).alpha'*estimates(j).alpha;
                    
                    try
                        if wishartPara == 1
                            estimates(j).iGamma = gamrnd(r0 + SubjectId(j)/2,(1/s0 + 0.5*estimates(j).alpha(:,g)'*estimates(j).alpha(:,g))^(-1));                            
                        else
                            estimates(j).iGamma = wishrnd(inv(h0*H_0 + H1),h0+SubjectId(j));
                        end
                    catch
                        prop_corrections = prop_corrections+1;
                        iHnew = inv(h0*H_0 + H1);
                        iHnew = triu(iHnew)+triu(iHnew)'+diag(diag(iHnew));
                        estimates(j).iGamma = wishrnd(iHnew,h0+SubjectId(j));
                        disp(['Iter: ', num2str(m),' Correction applied!']);
                    end

                    Gammaj = inv(estimates(j).iGamma);
                    estimates(j).rho = diag(Gammaj)'./(diag(Gammaj)'+(estimates(j).isigma2.^(-1)));
                    
                    %----- alpha ---------
                    iDj = diag([roi.numVoxels].*[estimates(j).ilambda2]);
                    Psi = inv(estimates(j).iGamma + iDj);
                    
                    if ~isempty(X_Covariates)
                        r_cov_reg=estimates(j).rmean*X_Covariates(stimulus(j).subjects,:)'; %%% G by Q  times Q by Kj then transpose... to Kj x G
                    else
                        r_cov_reg=0;
                    end                    
                    
                    rij_bar = (mBigj(j).B - repmat([estimates(j).theta], SubjectId(j), 1)) - r_cov_reg'; % Kj by G
                    try
                        estimates(j).alpha = mvnrnd((Psi*(iDj*rij_bar'))',Psi); % generates data for all subjects (via the mean MATRIX)
                    catch
                        prop_corrections2 = prop_corrections2+1;
                        Psi = triu(Psi)+triu(Psi)'+diag(diag(Psi));
                        estimates(j).alpha = mvnrnd((Psi*(iDj*rij_bar'))',Psi);
                    end
                    % store chains for (j)
                    if rem(m,thin_factor) == 0 && m > bayesSpatialModel.monteCarloParams.burnIn
                        post_sim1(j).iGamma(:,save_iter) = reshape(estimates(j).iGamma,numel(estimates(j).iGamma),1);
                        post_sim1(j).theta(:,save_iter) = estimates(j).theta;
                        post_sim1(j).rho(:,save_iter) = estimates(j).rho;
                        post_sim2(j).mu(:,save_iter) = mu;
                        post_sim2(j).alpha(:,save_iter) = reshape(estimates(j).alpha,SubjectId(j)*NumRegions,1); % should sort 1) by subject and 2) then by region, e.g. (1,1)...(1,NumRegions),(2,1),...,(2,NumRegions),...(K,NumRegions)
                        
                        if ~isempty(X_Covariates)
                            %post_sim2(j).r(:,:,save_iter)=parms1(j).r;
                            post_sim2(j).rmean(:,:,save_iter)=estimates(j).rmean; % G by Q
                        end                    
                    end
                end
            end           
        end     
    end
end
