function [ cont_sim, pp, post_sim1 ] = calc_contrasts(bayesSpatial, contrast_cell)
 
% CALC_CONTRASTS calculate the contrasts based on the Bayes Spatial Model
% Usage: [cont_sim, pp, post_sim1 ] = calc_contrasts(bayesSpatial,contrast_cell)
% Input: 
%       bayesSpatial: Bayes model parameter learned from MCMC with Gibbs sampling;
%       contrast_cell: contrast set up from design matrix;
% Output: 
%       cont_sim: contrast simulation on region and voxel level;
%       pp: posterior probability;
%       post_sim1: posterior simulation; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Lijun Zhang, Ph.D                        %
% Center for Biomedical Imaging Statistics (CBIS)  %
% Dept. of Biostatistics and Bioinformatics        %
% Emory University                                 %
% Atlanta, GA, 30322                               %
% Email: l.zhang@emory.edu                         %
% http://www.sph.emory.edu/bios/CBIS/about.html    %
% Jan, 2009, Last revision: Dec. 1, 2011           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

post_sim1 = bayesSpatial.PostSim1;
post_sim2 = bayesSpatial.PostSim2;

num_stimulus = bayesSpatial.NumStimulus;
num_regions = bayesSpatial.G;
subject = bayesSpatial.Subject;

 contrasts = contrast_cell{1,2};

% voxel estimates
 cont_sim.voxel = 0;
 for ii=1:size(contrasts, 2)
     cont_sim.voxel = cont_sim.voxel  + contrasts(ii) * post_sim2(ii).mu;
 end


% regional - use theta
cont_sim.regional = 0;
for ii=1:size(contrasts, 2)
    cont_sim.regional = cont_sim.regional  + contrasts(ii) * post_sim1(ii).theta;
end


% BAYES FACTORS - posterior odds vs prior odds...
%-------- mu -------------%
% probability that contrast is greater than zero, and divides
% by probability that contrast is less than zero
cont_sim.bf_voxel = sum(cont_sim.voxel >= 0,2)./max(1e-4,sum(cont_sim.voxel < 0,2));

cont_sim.bf_regional = sum(cont_sim.regional >= 0,2)./max(1e-4,sum(cont_sim.regional < 0,2));


% CONTRAST POSTERIOR PROBABILITIES OF Ho: theta>=0 vs H1: theta<0
M1 = size(cont_sim.voxel, 2);
pp.voxel = sum(cont_sim.voxel >= 0,2)./M1;

M2 = size(cont_sim.regional,2);
pp.regional = sum(cont_sim.regional >= 0,2)./M2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Here can save out the result for later debug
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% % Compute posterior me(di)ans
% pmean   = zeros(0, num_stimulus);
% pmedian = zeros(0, num_stimulus);
% 
% for j=1:num_stimulus
%     pmean(j).mu      = mean(post_sim2(j).mu,2);
%     pmean(j).sigma2g = mean((post_sim1(j).isigma2).^(-1),2);
%     pmean(j).alpha   = reshape(mean(post_sim2(j).alpha,2), num_regions, subject(j))';
% 
%     pmedian(j).mu       = median(post_sim2(j).mu,2);
%     pmedian(j).sigma2g  = median((post_sim1(j).isigma2).^(-1),2);
%     pmedian(j).alpha    = reshape(median(post_sim2(j).alpha,2), num_regions, subject(j))';
% end
% 
% cont_pmean.voxel = mean(post_sim2(2).mu - post_sim2(1).mu,2); % Trt difference for patients
% 
% cont_pmean.regional = mean(post_sim1(2).theta - post_sim1(1).theta,2); % Trt difference for patientsprop_corrections

%save('results/pp_results.mat', 'pp');


