function scan_data(C,map_regions,headInfo,save_path)
% This function reads in data that were obtained from stage-I estimates
% from the stop task of the cocaine-dependence study.
% The data include two sessions for both cocaine-dependent subjects and
% healthy controls.  In the case of the cocaine-dependent subjects, the
% sessions correspond to pre- and post-treatment periods.
%
% INPUTS:
%  data_info = text file containing 4 columns: (1)file location, (2)file name, 
%               (3)group id, (4)session/stimulus id
%  regions_map = map of brain regions (in analyze format); should be same dim as the data (e.g. [91 109 91] )
%
% OUTPUTS:
%   Y   =  response data for dependent variable 
%   X   =  design matrix
% brain_locs - location of brain voxels
% regions_map - map of the brain regions 
%
% Sample Usage:
%  data_info= '/home/prj/cbis/software_tools/data/coc_dep/smoothed/nifti/file_list.txt'
%  regions_map='anatomical/raal.nii';
%  save_path= '/home/prj/cbis/software_tools/data/coc_dep/smoothed/coc_dep_data.mat';
%  [Y, X, brain_locs, map_regions]=scan_data(data_info,regions_map,0,save_path, false);
%__________________________________________________________________________
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gordana Derado, Nov. 2008 (modifed from D. Bowman, 2006)
% Sanjay Agravat, Dec. 2008 (modifed from Gordana Derado, 2008)
% Modified by: Lijun Zhang, Ph.D                   %
% Center for Biomedical Imaging Statistics (CBIS)  %
% Dept. of Biostatistics and Bioinformatics        %
% Emory University                                 %
% Atlanta, GA, 30322                               %
% Email: l.zhang@emory.edu                         %
% http://www.sph.emory.edu/bios/CBIS/about.html    %
% Jan, 2009, Last revision: Jan 25, 2010           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% http://www.sph.emory.edu/bios/CBIS/about.html

global selectedRegionMap;
global AAL_template brodmann_template struct_template MNI_template;

% RegionMapType = selectedRegionMap;


h = waitbar(0,'Accessing scan information   ...   Please wait ...  ');

scans=size(C{1},1);
column = length(headInfo);
%----- design matrix ---------

groups=unique(C{3});
no_groups=numel(groups);

sessions=unique(C{4});
no_sessions=numel(sessions);

stimuli=unique(C{5});
no_stimuli=numel(stimuli);

X=zeros(scans,no_groups * no_sessions * no_stimuli);
for g=1:no_groups
    indx1= strcmp(C{3},groups(g));  % find index for group (if C{3}(i)=='g') 
    ind1=zeros(scans,1); 
    ind1(indx1)=1;
    for s=1:no_sessions;
        indx2= strcmp(C{4},sessions(s)); % find index for session (if C{4}(i)=='s')
        ind2=zeros(scans,1);
        ind2(indx2)=1;
        for t=1:no_stimuli
            indx3= strcmp(C{5},stimuli(t)); % find index for stimuli (if C{5}(i)=='t')
            ind3=zeros(scans,1);
            ind3(indx3)=1;
            pos=sub2ind([no_stimuli no_sessions no_groups], t, s, g); % Matlab function
            X(:,pos)=ind1 .* ind2 .* ind3;                    
        end
    end;
end;

%%%%Current keep coviant separately
X_Covariates = [];
if column > 5
    for kk = 1:column-5
        clear temp no_temp indT
        temp = unique(C{kk+5});
        tempCol = C{kk+5};
        for numR = 1:scans
            tempR = str2num(tempCol{numR});
            if isempty(tempR)
                X_Covariates(numR, kk) = strmatch(tempCol{numR},temp);
            else
                X_Covariates(numR, kk) = tempR;
            end
        end
    end
end

%----------------------------------------------------------

for i = 1:scans
    newtitle=['Reading data from image ' num2str(i)];
    waitbar(i/scans,h,newtitle)
    data_path=fullfile(char(C{1}(i)),char(C{2}(i)));
    data1 = load_nii(data_path);
    if i==1
        im_dim=size(data1.img);
        voxels = prod(im_dim);
     end;
    data(i,1:voxels) = single(reshape(data1.img,1,voxels));
end

clear data1;
close(h);

% -- read in the regions map ------

prodtemp=prod(abs(data));
brain_ind = (prodtemp>0);

mask_ind=reshape((map_regions>0),1,voxels);
brain_locs=find(mask_ind.*brain_ind);

Y=data(:,brain_locs);

% if save_results
    if nargin<4
        save_path='data/analysis_data.mat';
    end;
     save (save_path, '-v7.3', 'Y', 'X', 'X_Covariates', 'brain_locs', 'map_regions', 'C', 'headInfo', ...
         'selectedRegionMap',  'AAL_template', 'brodmann_template', 'struct_template', 'MNI_template');
% end