function connectivity_3D_SBHM(f, conn_matrix, all_regions, brain_map, ROIs, region_ids, save_path, ...
                                thresh, movie_name, alpha, line_thick, fig_title);
% This function creates a 3D figure and a corresponding movie
% for visualization of brain regions and their connectivity
% It assumes a standard brain atlas is used (e.g. brodmann, aal)
% If you want to use your own regions (saved as nifti), see
% connectivity_3D_ROI.m
% probability of regions being connected (from conn_matrix)
% is indicated by the connecting line thickness
%------------
% Input: 
% conn_matrix_path = full path to connectivity matrix (in .txt format)
% brain_map = anatomical parcelation map (aal, brodmann)
% ROIs=string list of region names
% region_ids=list of region ids (cell array)
% save_path = where you want to save the results
% thresh = thereshold for the connectivity to be displayed
% movie_name = name of the movie that will be generated (avi and mpg)
% alpha = transparency parameter, btw. 0 and 1, lower = more transparent
% line_thick = multiplication factor for line thickness
% fig_title = figure title
%--------------
% Output: none
%______________
% Sample usage:
% conn_matrix_path='S:\CBIS\projects\misc_code\visualization\conn.txt';
% brain_map='S:\CBIS\data\anatomical\rbrodmann_2mm';
% save_path='S:\CBIS\projects\misc_code\visualization\';
% ROIs={'BA 8','BA 9','BA 10','BA 24/32'};
% region_ids={[8],[9],[10],[24 32]};
% thresh= 0.3;
% movie_name='SBTM_1';
% connectivity_3D(conn_matrix_path,brain_map,ROIs,region_ids,save_path,thresh,movie_name);
%_________________________________
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Writen by: Gordana Derado, Ph.D, Emory University%
% Modified by: Lijun Zhang, Ph.D                   %
% Center for Biomedical Imaging Statistics (CBIS)  %
% Dept. of Biostatistics and Bioinformatics        %
% Emory University                                 %
% Atlanta, GA, 30322                               %
% Email: l.zhang@emory.edu                         %
% http://www.sph.emory.edu/bios/CBIS/about.html    %
% Jan, 2009, Last revision: Jan 25, 2010           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%close all


disp(ROIs);
disp(region_ids);

home=pwd;

if nargin<8;
    thresh=0.3;
end;
if nargin<9;
    movie_name='my_movie';
end;
if nargin<10;
    alpha=0.5;
end;
if nargin<11;
    line_thick=5;
end;
if nargin<12;
    fig_title='Node to node connectivity';
end;

%----- load structural brain  ----
% NOTE: this is hard coded, need to change if user provides path to structural
% image; or if image is located somewhere else
struct_brain_path='anatomical/MNI152_T1_2mm_brain.nii';
i=load_nii(struct_brain_path);
data=double(i.img);

fv=isosurface(data,37);
p=patch(fv);
set(p,'FaceColor',[0.9 0.6 0],'EdgeColor','none','FaceAlpha',0.3);  % smaller value of FaceAlpha = more transparent
daspect([1 1 1])
view(3); axis tight % need view(3) for 3d display
lighting gouraud
view(45,25);

%---- load the brain map (e.g. aal, brodmann) -----
map=load_nii(brain_map);
map=map.img;

%-- display the regions ---
nr=size(region_ids,1); % number of regions
color_map=jet(nr);
H=[]; % handles for surfaces
ii = 1;
indices = [];
horzRegions = region_ids';
for i=1:size(all_regions, 2);

    
    if (ii <= size(region_ids,1)) && (all_regions{i} == horzRegions{ii}(1))
        reg_indx=find(ismember(map,region_ids{ii}));
        im=zeros(size(data));
        im(reg_indx)=1;
        
        fv=isosurface(im);
        p=patch(fv);
        hold on;

        set(p,'FaceColor',color_map(nr+1-ii,:),'EdgeColor','none','FaceAlpha',alpha);  % smaller value of FaceAlpha = more transparent
        daspect([1 1 1])
        view(3); axis tight % need view(3) for 3d display
        lighting gouraud
        H=[H p];  % save handles of surfaces for legend;

        %------- find region centeroid -------
        r=find(im>0);
        [I,J,K]=ind2sub([91 109 91],r);
        ry_vec=reshape(I,numel(I),1); % x and y switched in MATLAB graphics  
        rx_vec=reshape(J,numel(J),1); % x and y switched in MATLAB graphics
        rz_vec=reshape(K,numel(K),1);
        locs_mean_g=mean([rx_vec ry_vec rz_vec]);
        Df=([rx_vec ry_vec rz_vec] - repmat(locs_mean_g,numel(r),1))';
        d=sqrt(sum(Df.*Df));
        mloc=find(d==min(d));
        rx(i)=rx_vec(mloc);
        ry(i)=ry_vec(mloc);
        rz(i)=rz_vec(mloc);
        
        indices(i) = i;
        ii = ii + 1;
    else
        indices(i) = 0;
    end
    
end;

%-- load connectivity matrix 
conn = conn_matrix;
%-- add connecting lines ----
for i=1:size(conn,2);
    c = conn{i};
    for j=i+1:size(c,1);
        if  (indices(i) > 0) && (indices(j) > 0) && ( c(indices(i), indices(j)) > 0 ) 
            line([rx(indices(i)) rx(indices(j))],[ry(indices(i)) ry(indices(j))], ...
                [rz(indices(i)) rz(indices(j))], ...
                'Marker','.','Color','black','LineStyle','-','LineWidth', ...
                  line_thick*c(indices(i), indices(j)));
        end
    end
    % comment out if don't want region labesl to be displayed
end;
title(fig_title); legend(H,ROIs);

%----- make movie ---------------
%fig1=figure(1); 
fig1=gcf;
% Resize figure window to size of movie required. 
winsize = get(fig1,'Position');     % Record the size of the plot window
winsize(1:2) = [0 0];               % Adjust size of this window to include the whole figure window 
                                    % (if you require the axes, title and
                                    % axis labels in the movie)
numframes=72;                       % Set the number of frames
A=moviein(numframes,fig1,winsize);  % Create a MATLAB movie matrix
set(fig1,'NextPlot','replacechildren')  % Fix the features of the plot window (ensures each frame of the movie is the same size)
% Within a loop, plot each picture and save to MATLAB movie matrix:

% also create an avi file
mov = avifile([save_path movie_name '.avi'],'fps',5,'quality',100,'compression', 'Cinepak'); % Use "Cinepak' compression method
for i=1:numframes 
   view(5*i+40,25) % rotate figure 
   % add axis label, legends, titles, etc. in here 
   A(:,i)=getframe(fig1,winsize);
   mov = addframe(mov,A(:,i));
end 
mov = close(mov);
clear mex;
view(45,20);

cd(home);

end

function figClose(src, evt)
    disp('closing');
    clear mex;
    delete(gcf);
end