function connectivity_3D_plot(conn_matrix, indices, brain_map, region_struct, ROIs, region_ids, ...
                                threshold, alpha, line_thick, viewAZ, viewEL)
% This function creates a 3D figure and a corresponding movie
% for visualization of brain regions and their connectivity
% It assumes a standard brain atlas is used (e.g. brodmann, aal)
% If you want to use your own regions (saved as nifti), see
% connectivity_3D_ROI.m
% probability of regions being connected (from conn_matrix)
% is indicated by the connecting line thickness
%------------
% Input: 
% conn_matrix_path = full path to connectivity matrix (in .txt format)
% brain_map = anatomical parcelation map (aal, brodmann)
% ROIs=string list of region names
% region_ids=list of region ids (cell array)
% thresh = thereshold for the connectivity to be displayed
% movie_name = name of the movie that will be generated (avi and mpg)
% alpha = transparency parameter, btw. 0 and 1, lower = more transparent
% line_thick = multiplication factor for line thickness
%--------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Writen by: Gordana Derado, Ph.D, Emory University%
% Modified by: Lijun Zhang, Ph.D                   %
% Center for Biomedical Imaging Statistics (CBIS)  %
% Dept. of Biostatistics and Bioinformatics        %
% Emory University                                 %
% Atlanta, GA, 30322                               %
% Email: l.zhang@emory.edu                         %
% http://www.sph.emory.edu/bios/CBIS/about.html    %
% Jan, 2009, Last revision: Jan 25, 2010           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% http://www.sph.emory.edu/bios/CBIS/about.html


brain_template = brain_map.struct_brain;

fv=isosurface(brain_template,10);
background=patch(fv);
set(background,'FaceColor',[0.7 0.7 0.7],'EdgeColor','none','FaceAlpha',0.3);  % smaller value of FaceAlpha = more transparent
daspect([1 1 1])
view(3); axis tight % need view(3) for 3d display
lighting gouraud
view(45,25);

%---- load the brain map (e.g. aal, brodmann) -----
map1 = region_struct.img;
map = flipdim(map1, 1);

%-- display the regions ---
nr=size(region_ids,1); % number of regions
color_map=jet(nr);
H=[]; % handles for surfaces
ii = 1;
%indices = [];
Id = find(indices);
rx = zeros(1, length(indices));
ry = zeros(1, length(indices));
rz = zeros(1, length(indices));

for i=1:size(region_ids, 1);    
        reg_indx=find(ismember(map,region_ids{i}));
        im=zeros(size(brain_template));
        im(reg_indx)=1;
        
        fv=isosurface(im);
        p=patch(fv);
        hold on;

        set(p,'FaceColor',color_map(nr+1-i,:),'EdgeColor','none','FaceAlpha',alpha);  % smaller value of FaceAlpha = more transparent
        daspect([1 1 1])
        view(3); axis tight % need view(3) for 3d display
        lighting gouraud
        H=[H p];  % save handles of surfaces for legend;

        %------- find region centeroid -------
        r=find(im>0);
        [I,J,K]=ind2sub(size(map),r);
        % ry_vec=reshape(I,numel(I),1); % x and y switched in MATLAB graphics  
        % rx_vec=reshape(J,numel(J),1); % x and y switched in MATLAB graphics
        % rz_vec=reshape(K,numel(K),1);
        
        ry_vec=I; % x and y switched in MATLAB graphics  
        rx_vec=J; % x and y switched in MATLAB graphics
        rz_vec=K;
        
        locs_mean_g=mean([rx_vec ry_vec rz_vec]);
        Df=([rx_vec ry_vec rz_vec] - repmat(locs_mean_g,numel(r),1))';
        d=sum(Df.*Df);
        mloc=min(find(d==min(d)));
        rx(Id(i))=rx_vec(mloc);
        ry(Id(i))=ry_vec(mloc);
        rz(Id(i))=rz_vec(mloc);
end;

for i=1:size(conn_matrix,1);
    for j=i+1:size(conn_matrix,2);
        if  (indices(i) > 0) && (indices(j) > 0) && ( conn_matrix(indices(i), indices(j)) > threshold ) 
            line([rx(indices(i)) rx(indices(j))],[ry(indices(i)) ry(indices(j))], ...
                [rz(indices(i)) rz(indices(j))], ...
                'Marker','.','Color','black','LineStyle','-','LineWidth', ...
                  line_thick*conn_matrix(indices(i), indices(j)));
        end
    end
    % comment out if don't want region labesl to be displayed
    % text('Position',[rx(i)+1 ry(i)+1 rz(i)+1],'String',ROIs(i),'Color','black');
end;

legend_h = legend(H,ROIs);
set(legend_h, 'TextColor', 'k', 'Location', 'Best','FontSize', 8);
%legend boxoff,

view(viewAZ, viewEL);
end
